package compbio.ws.execstat;

import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.Date;
import java.util.List;

import compbio.ws.client.Services;

public class StatProcessor {

	List<JobStat> stats;

	StatProcessor(List<JobStat> stats) {
		this.stats = stats;
	}

	List<JobStat> getAbandonedJobs() {
		List<JobStat> abJobs = new ArrayList<StatProcessor.JobStat>();
		for (JobStat js : stats) {
			if (!js.isCollected) {
				abJobs.add(js);
			}
		}
		return abJobs;
	}

	List<JobStat> getCancelledJobs() {
		List<JobStat> abJobs = new ArrayList<StatProcessor.JobStat>();
		for (JobStat js : stats) {
			if (js.isCancelled) {
				abJobs.add(js);
			}
		}
		return abJobs;
	}

	List<JobStat> sortByRuntime() {
		List<JobStat> abJobs = new ArrayList<StatProcessor.JobStat>(stats);
		Collections.sort(abJobs, JobStat.RUNTIME);
		return abJobs;
	}

	List<JobStat> sortByStartTime() {
		List<JobStat> abJobs = new ArrayList<StatProcessor.JobStat>(stats);
		Collections.sort(abJobs, JobStat.STARTTIME);
		return abJobs;
	}

	List<JobStat> sortByResultSize() {
		List<JobStat> abJobs = new ArrayList<StatProcessor.JobStat>(stats);
		Collections.sort(abJobs, JobStat.RESULTSIZE);
		return abJobs;
	}

	int getJobNumber() {
		return stats.size();
	}

	public StatProcessor getSingleWSStat(Services webService) {
		List<JobStat> wsStat = new ArrayList<StatProcessor.JobStat>();
		for (JobStat js : stats) {
			if (js.webService == webService) {
				wsStat.add(js);
			}
		}
		return new StatProcessor(wsStat);
	}

	long getTotalRuntime() {
		long counter = 0;
		for (JobStat js : stats) {
			int jobtime = js.getRuntime();
			if (jobtime != ExecutionStatCollector.UNDEFINED) {
				counter += jobtime;
			}
		}
		return counter;
	}

	List<JobStat> getUnsuccessfulJobs() {
		List<JobStat> aJobs = new ArrayList<StatProcessor.JobStat>();
		for (JobStat js : stats) {
			int jobtime = js.getRuntime();
			if (js.resultSize == ExecutionStatCollector.UNDEFINED) {
				aJobs.add(js);
			}
		}
		return aJobs;
	}

	public String reportStat() {
		String report = "Total Jobs: " + getJobNumber() + "\n";
		report += "Abandoned Jobs: " + getAbandonedJobs().size() + "\n";
		report += "Cancelled Jobs: " + getCancelledJobs().size() + "\n";
		report += "Total Runtime (s): " + getTotalRuntime() + "\n";
		report += "Unsuccessful Jobs: " + getUnsuccessfulJobs().size() + "\n";
		report += "10 longest jobs: \n\n" + sortByRuntime().subList(0, 9)
				+ "\n";
		report += "10 biggest jobs: \n\n" + sortByResultSize().subList(0, 9)
				+ "\n";
		return report;
	}

	static class JobStat {

		static final Comparator<JobStat> RUNTIME = new Comparator<JobStat>() {
			@Override
			public int compare(JobStat o1, JobStat o2) {
				return new Integer(o2.getRuntime()).compareTo(o1.getRuntime());
			}
		};

		static final Comparator<JobStat> STARTTIME = new Comparator<JobStat>() {
			@Override
			public int compare(JobStat o1, JobStat o2) {
				return new Long(o1.start).compareTo(o2.start);
			}
		};

		static final Comparator<JobStat> RESULTSIZE = new Comparator<JobStat>() {
			@Override
			public int compare(JobStat o1, JobStat o2) {
				return new Long(o2.resultSize).compareTo(o1.resultSize);
			}
		};

		Services webService;
		String jobname;
		long start;
		long finish;
		long inputSize;
		long resultSize;
		boolean isCollected;
		boolean isCancelled;

		JobStat(Services webService, String jobname, long start, long finish,
				long inputSize, long resultSize, boolean isCollected,
				boolean isCancelled) {
			super();
			this.webService = webService;
			this.jobname = jobname;
			this.start = start;
			this.finish = finish;
			this.inputSize = inputSize;
			this.resultSize = resultSize;
			this.isCollected = isCollected;
			this.isCancelled = isCancelled;
		}

		@Override
		public int hashCode() {
			final int prime = 31;
			int result = 1;
			result = prime * result + (int) (finish ^ (finish >>> 32));
			result = prime * result + (int) (inputSize ^ (inputSize >>> 32));
			result = prime * result + (isCancelled ? 1231 : 1237);
			result = prime * result + (isCollected ? 1231 : 1237);
			result = prime * result
					+ ((jobname == null) ? 0 : jobname.hashCode());
			result = prime * result + (int) (resultSize ^ (resultSize >>> 32));
			result = prime * result + (int) (start ^ (start >>> 32));
			return result;
		}

		@Override
		public boolean equals(Object obj) {
			if (this == obj)
				return true;
			if (obj == null)
				return false;
			if (getClass() != obj.getClass())
				return false;
			JobStat other = (JobStat) obj;
			if (finish != other.finish)
				return false;
			if (inputSize != other.inputSize)
				return false;
			if (isCancelled != other.isCancelled)
				return false;
			if (isCollected != other.isCollected)
				return false;
			if (jobname == null) {
				if (other.jobname != null)
					return false;
			} else if (!jobname.equals(other.jobname))
				return false;
			if (resultSize != other.resultSize)
				return false;
			if (start != other.start)
				return false;
			return true;
		}

		public int getRuntime() {
			if (start != ExecutionStatCollector.UNDEFINED
					&& finish != ExecutionStatCollector.UNDEFINED) {
				return (int) (finish - start) / 1000;
			}
			return ExecutionStatCollector.UNDEFINED;
		}

		@Override
		public String toString() {
			return getJobReport();
		}

		String getJobReport() {
			String report = "WS: " + webService + "\n";
			report += "JOB: " + jobname + "\n";
			if (start != ExecutionStatCollector.UNDEFINED) {
				report += "Started " + new Date(start) + "\n";
			}
			if (finish != ExecutionStatCollector.UNDEFINED) {
				report += "Finished " + new Date(finish) + "\n";
			}
			if (start != ExecutionStatCollector.UNDEFINED
					&& finish != ExecutionStatCollector.UNDEFINED) {
				report += "Runtime " + getRuntime() + "\n";
			}
			report += "Input size " + inputSize + "\n";
			report += "Result size " + resultSize + "\n";
			report += "Collected? " + isCollected + "\n";
			return report;
		}

		/**
		 * Header Job Started Finished Runtime Input Result
		 */
		String getJobReportTabulated() {
			String report = webService + "\t";
			report += jobname + "\t";
			if (start != ExecutionStatCollector.UNDEFINED) {
				report += ExecutionStatCollector.DF.format(new Date(start))
						+ "\t";
			} else {
				report += ExecutionStatCollector.UNDEFINED + "\t";
			}
			if (finish != ExecutionStatCollector.UNDEFINED) {
				report += ExecutionStatCollector.DF.format(new Date(finish))
						+ "\t";
			} else {
				report += ExecutionStatCollector.UNDEFINED + "\t";
			}
			if (start != ExecutionStatCollector.UNDEFINED
					&& finish != ExecutionStatCollector.UNDEFINED) {
				report += getRuntime() + "\t";
			} else {
				report += ExecutionStatCollector.UNDEFINED + "\t";
			}
			report += inputSize + "\t";
			report += resultSize + "\t";
			report += isCollected + "\t";
			return report;
		}

	}

}
