package compbio.ws.execstat;

import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.PreparedStatement;
import java.sql.SQLException;
import java.sql.Statement;
import java.sql.Timestamp;

import compbio.ws.execstat.StatProcessor.JobStat;

public class StatWriter {

	/* the default framework is embedded */
	// private final String framework = "embedded";
	private static final String driver = "org.apache.derby.jdbc.EmbeddedDriver";
	private static final String protocol = "jdbc:derby:";
	private static final String statDBName = "ExecutionStatistic";

	static Connection getDBConnection() throws SQLException {
		// TODO
		System.setProperty("derby.system.home", ".");

		Connection conn = DriverManager.getConnection(protocol + statDBName
				+ ";create=true");

		// We want to control transactions manually. Autocommit is on by
		// default in JDBC.
		conn.setAutoCommit(true);
		return conn;
	}

	// ServiceName,jobname,start,finish,inputSize,resultSize,isCancelled,isCollected
	static void createStatTable() throws SQLException {
		Connection conn = getDBConnection();
		/*
		 * Creating a statement object that we can use for running various SQL
		 * statements commands against the database.
		 */
		Statement s = conn.createStatement();
		String create = "create table exec_stat("
				+ "number INT GENERATED ALWAYS AS IDENTITY,"
				+ "service_name VARCHAR(15) NOT NULL, "
				+ "job_id VARCHAR(35) NOT NULL PRIMARY KEY, "
				+ "start TIMESTAMP," + "finish TIMESTAMP,"
				+ "inputsize BIGINT," + "resultsize BIGINT,"
				+ "isCancelled SMALLINT NOT NULL,"
				+ "isCollected SMALLINT NOT NULL)";
		// We create a table...
		System.out.println(create);
		s.execute(create);
		s.close();
		conn.close();
	}

	static void insertData(JobStat jobstatus) throws SQLException {
		Connection conn = getDBConnection();
		String insert = "insert into exec_stat (service_name, job_id, start, finish, "
				+ "inputsize, resultsize, isCancelled, isCollected) "
				+ "VALUES (?, ?, ?, ?, ?, ?, ?, ? )";
		PreparedStatement pstm = conn.prepareStatement(insert);

		pstm.setString(1, "webservice");
		pstm.setString(2, "@Clustal#980273495452357");
		pstm.setTimestamp(3, new Timestamp(190385934834l));
		pstm.setTimestamp(4, new Timestamp(190332423423l));
		pstm.setLong(5, 1232);
		pstm.setLong(6, 1432422);
		pstm.setShort(7, (short) 1);
		pstm.setShort(8, (short) 0);
		pstm.executeUpdate();
		pstm.close();
		conn.close();
	}

	void shutdownDBServer() {
		// ## DATABASE SHUTDOWN SECTION ##
		/***
		 * In embedded mode, an application should shut down Derby. Shutdown
		 * throws the XJ015 exception to confirm success.
		 ***/
		boolean gotSQLExc = false;
		try {
			DriverManager.getConnection("jdbc:derby:;shutdown=true");
		} catch (SQLException se) {
			if (se.getSQLState().equals("XJ015")) {
				gotSQLExc = true;
			}
		}
		if (!gotSQLExc) {
			System.out.println("Database did not shut down normally");
		} else {
			System.out.println("Database shut down normally");
		}
	}
	public static void main(String[] args) throws SQLException {
		// createStatTable();
		insertData(null);
	}
}
